@@ -45,12 +45,7 @@
 #include <netdb.h>
 #endif /* HAVE_NETDB_H */
 
-#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
-#endif /* HAVE_FCNTL_H */
-#ifdef HAVE_SYS_FCNTL_H
-#include <fcntl.h>
-#endif /* HAVE_SYS_FCNTL_H */
 
 #include <termios.h>
 #include <sys/ioctl.h>
@@ -65,6 +60,7 @@ int	debug = 0;
 char	*bad = "Bad request!";
 char	buf[ 4624 ];
 char	*server;
+static char	*version = VERSION;
 
 void usage( char *p )
 {
@@ -118,7 +114,7 @@ int main( int ac, char **av )
     }
     server = hostname;
 
-    while (( c = getopt( ac, av, "dln:" )) != EOF ) {
+    while (( c = getopt( ac, av, "dlVvn:" )) != EOF ) {
 	switch ( c ) {
 	case 'd' :
 	    debug++;
@@ -129,6 +125,14 @@ int main( int ac, char **av )
 	case 'n' :
 	    server = optarg;
 	    break;
+    case 'V' :
+    case 'v' :
+        fprintf(stdout, "timelord %s - Timelord Time Server Daemon\n"
+               "Copyright (c) 1990,1992 Regents of The University of Michigan.\n"
+               "\tAll Rights Reserved.\n"
+               "Copyright (c) 1990, The University of Melbourne.\n", version );
+        exit ( 1 );
+        break;
 	default :
 	    fprintf( stderr, "Unknown option -- '%c'\n", c );
 	    usage( *av );
