@@ -1,6 +1,9 @@
 #ifndef ATALKD_MAIN_H
 #define ATALKD_MAIN_H
 
+#define _PATH_ATALKDEBUG	"/tmp/atalkd.debug"
+#define _PATH_ATALKDTMP		"atalkd.tmp"
+
 #include <sys/types.h>
 #include "config.h"
 
@@ -8,13 +11,9 @@ extern int transition;
 extern int stabletimer, newrtmpdata;
 
 int ifconfig ( const char *, unsigned long, struct sockaddr_at * );
-void setaddr ( struct interface *, u_int8_t, u_int16_t,
-        u_int8_t, u_int16_t, u_int16_t );
+void setaddr ( struct interface *, uint8_t, uint16_t,
+        uint8_t, uint16_t, uint16_t );
 void bootaddr ( struct interface * );
 void dumpconfig ( struct interface * );
 
-#ifdef linux
-int ifsetallmulti ( const char *, int);
-#endif
-
 #endif /* ATALKD_MAIN_H */
